import threading



class AsyncUserInput(threading.Thread):
    """
    ### A class that runs as a separate thread for capturing user input asynchronously.

    #### Usage:
    ```py
        async_input = AsyncInput(input_cbk=callback_function).
        async_input.pause() # (WARNING!: Experimental could cause Problems.) Pauses until resumed.
        async_input.resume() # Resume the input thread.
        async_input.start() # (WARNING!: This is already done automatically).
    ```
    #### Args:
    ```py
        input_cbk: Callback function to be invoked when input is received.
    ```
    """

    def __init__(self, input_callback, input_function = input, name: str = 'AsyncUserInput-thread'):
        self.input_callback = input_callback
        self.input_function = input_function
        self.paused         = False
        self.pause_cond     = threading.Condition(threading.Lock())
        super(AsyncUserInput, self).__init__(name=name)
        self.start()

    def run(self):
        while True:
            with self.pause_cond:
                while self.paused:
                    self.pause_cond.wait()
                self.input_callback(self.input_function())

    def pause(self):
        self.paused = True
        self.pause_cond.acquire()
        
    def resume(self):
        self.paused = False
        self.pause_cond.notify()
        self.pause_cond.release()


import tkinter as tk
from tkinter import filedialog



def open_file(mode, **filedialogargs):
    root = tk.Tk()
    root.withdraw()
    try:
        if mode == "file":
            file_path = filedialog.askopenfile(**filedialogargs); return file_path
        elif mode == "file_name":
            file_path = filedialog.askopenfilename(**filedialogargs); return file_path
        elif mode == "files":
            file_path = filedialog.askopenfiles(**filedialogargs); return file_path
        elif mode == "file_names":
            file_path = filedialog.askopenfilenames(**filedialogargs); return file_path
        else: raise ValueError("mode was not 'file', 'file_name', 'files' or 'file_names'!")
    except Exception as e:
        raise e


def save_file(mode, **filedialogargs):
    root = tk.Tk()
    root.withdraw()
    try:
        if mode == "file":
            file_path = filedialog.asksaveasfile(**filedialogargs); return file_path
        elif mode == "file_name":
            file_path = filedialog.asksaveasfilename(**filedialogargs); return file_path
        elif mode == "files":
            file_path = filedialog.askopenfiles(**filedialogargs); return file_path
        elif mode == "file_names":
            file_path = filedialog.askopenfilenames(**filedialogargs); return file_path
        else: raise ValueError("mode was not 'file', 'file_name', 'files' or 'file_names'!")
    except Exception as e:
        raise e


def open_dir(**dirdialogargs):
    root = tk.Tk()
    root.withdraw()
    try:
        dir_path = filedialog.askdirectory(**dirdialogargs); return dir_path
    except Exception as e:
        raise e


from setuptools import setup, find_packages
import codecs
import os
from einfach import __version__

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = __version__
DESCRIPTION = 'A collection of useful code "snippets"'
LONG_DESCRIPTION = 'A collection of useful code "snippets" or "helpers" that implement little but annoying to add things for you'

# Setting up
setup(
    name="einfach",
    version=VERSION,
    author="rotgruengelb (Daniel)",
    author_email="<code@rotgruengelb.net>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION + "\n" + long_description,
    packages=find_packages(),
    license="MIT",
    url="https://github.com/rotgruengelb/einfach",
    project_urls={
    'Source': 'https://github.com/rotgruengelb/einfach',
    'Issues': 'https://github.com/rotgruengelb/einfach/issues',
    'Wiki': 'https://github.com/rotgruengelb/einfach/wiki/Features'
    },
    install_requires=[],
    keywords=['python', 'helper', 'helpers', 'help', 'easy code', 'ease of use', "snippets", "code snippets"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development",
        "Topic :: Software Development :: User Interfaces"
    ]
)


import sys, os



def clip(content: str, no_os_error: bool = False):
    if content.isspace() == False and not content == "":
        if sys.platform == "win32":
            try:
                os.system(f'echo {content} | clip')
            except Exception as e: raise e
        elif no_os_error == False:
            raise OSError("Currently only win32 systems are supported!")
    else:
        raise ValueError("Content can't be empty or only contain whitespace/spaces.")





def would_be_valid_float(value: str) -> bool:
    return True if str(value).replace('.', '', 1).isdigit() else False


def is_float_in_range(value: float|int, min_value: float|int, max_value: float|int) -> bool:
    return float(min_value) <= float(value) <= float(max_value)